import { Address, Cart, CheckoutRequestBody, CheckoutSelectors, Consignment, ConsignmentAssignmentRequestBody, Country, Customer, CustomerRequestOptions, FormField, ShippingInitializeOptions, ShippingRequestOptions } from '@bigcommerce/checkout-sdk'
import { noop } from 'lodash'
import React, { Component, ReactNode } from 'react'
import { createSelector } from 'reselect'
import { connect } from 'react-redux'

import B3Checkout from '../b3checkout'
import { isEqualAddress, mapAddressFromFormValues } from '../address'
import { withCheckout, CheckoutContextProps } from '../checkout'
import { EMPTY_ARRAY } from '../common/utility'
import { LoadingOverlay } from '../ui/loading'

import { UnassignItemError } from './errors'
import getShippableItemsCount from './getShippableItemsCount'
import getShippingMethodId from './getShippingMethodId'
import { MultiShippingFormValues } from './MultiShippingForm'
import ShippingForm from './ShippingForm'
import ShippingHeader from './ShippingHeader'
import { SingleShippingFormValues } from './SingleShippingForm'

export interface ShippingProps {
    cartHasChanged: boolean
    isMultiShippingMode: boolean
    onCreateAccount(): void
    onToggleMultiShipping(): void
    onReady?(): void
    onUnhandledError(error: Error): void
    onSignIn(): void
    navigateNextStep(billingSameAsShipping: boolean): void
}

export interface WithCheckoutShippingProps {
    billingAddress?: Address
    cart: Cart
    consignments: Consignment[]
    countries: Country[]
    countriesWithAutocomplete: string[]
    customer: Customer
    customerMessage: string
    googleMapsApiKey: string
    isGuest: boolean
    isInitializing: boolean
    isLoading: boolean
    isShippingStepPending: boolean
    methodId?: string
    shippingAddress?: Address
    shouldShowMultiShipping: boolean
    shouldShowOrderComments: boolean
    shouldValidateSafeInput: boolean
    assignItem(consignment: ConsignmentAssignmentRequestBody): Promise<CheckoutSelectors>
    deinitializeShippingMethod(options: ShippingRequestOptions): Promise<CheckoutSelectors>
    deleteConsignments(): Promise<Address | undefined>
    getFields(countryCode?: string): FormField[]
    initializeShippingMethod(options: ShippingInitializeOptions): Promise<CheckoutSelectors>
    loadShippingAddressFields(): Promise<CheckoutSelectors>
    loadShippingOptions(): Promise<CheckoutSelectors>
    signOut(options?: CustomerRequestOptions): void
    unassignItem(consignment: ConsignmentAssignmentRequestBody): Promise<CheckoutSelectors>
    updateBillingAddress(address: Partial<Address>): Promise<CheckoutSelectors>
    updateCheckout(payload: CheckoutRequestBody): Promise<CheckoutSelectors>
    updateShippingAddress(address: Partial<Address>): Promise<CheckoutSelectors>
}

interface ShippingState {
    isInitializing: boolean
}

interface B3Props {
    b3Company: any
    isB3User: boolean
}

class Shipping extends Component<B3Props & ShippingProps & WithCheckoutShippingProps, ShippingState> {
    constructor(props: B3Props & ShippingProps & WithCheckoutShippingProps) {
        super(props)

        this.state = {
            isInitializing: true,
        }
    }

    async componentDidMount(): Promise<void> {
        const {
            loadShippingAddressFields,
            loadShippingOptions,
            onReady = noop,
            onUnhandledError = noop,
        } = this.props

        try {
            await Promise.all([
                loadShippingAddressFields(),
                loadShippingOptions(),
            ])

            onReady()
        } catch (error) {
            onUnhandledError(error)
        } finally {
            this.setState({ isInitializing: false })
        }
    }

    get shouldShowSaveAddress() {
        const {
            isGuest,
            isB3User,
        } = this.props
        const {
            storage,
            constants,
        } = B3Checkout.utils

        return isB3User
            ? (storage.B3AddressBookActionPermission.getValue() === constants.B3AddressBookPermission.isAllow)
            : !isGuest
    }

    get addresses() {
        const {
            customer,
            isB3User,
            b3Company,
        } = this.props

        return isB3User
            ? b3Company.addresses.filter((address: { isShipping: number }) => address.isShipping === 1)
            : customer.addresses
    }

    render(): ReactNode {
        const {
            isGuest,
            shouldShowMultiShipping,
            customer,
            unassignItem,
            updateShippingAddress,
            initializeShippingMethod,
            deinitializeShippingMethod,
            isMultiShippingMode,
            onToggleMultiShipping,
            isB3User,
            b3Company,
            ...shippingFormProps
        } = this.props

        const {
            isInitializing,
        } = this.state

        return (
            <div className="checkout-form">
                <ShippingHeader
                    isGuest={isGuest}
                    isMultiShippingMode={isMultiShippingMode}
                    onMultiShippingChange={onToggleMultiShipping}
                    shouldShowMultiShipping={shouldShowMultiShipping}
                />

                <LoadingOverlay
                    isLoading={isInitializing}
                    unmountContentWhenLoading
                >
                    <ShippingForm
                        {...shippingFormProps}
                        addresses={this.addresses}
                        deinitialize={deinitializeShippingMethod}
                        initialize={initializeShippingMethod}
                        isGuest={isGuest}
                        isMultiShippingMode={isMultiShippingMode}
                        onMultiShippingSubmit={this.handleMultiShippingSubmit}
                        onSingleShippingSubmit={this.handleSingleShippingSubmit}
                        onUseNewAddress={this.handleUseNewAddress}
                        shouldShowSaveAddress={this.shouldShowSaveAddress}
                        updateAddress={updateShippingAddress}
                    />
                </LoadingOverlay>
            </div>
        )
    }

    private handleSingleShippingSubmit: (values: SingleShippingFormValues) => void = async ({
        billingSameAsShipping,
        shippingAddress: addressValues,
        orderComment,
    }) => {
        const {
            customerMessage,
            updateCheckout,
            updateShippingAddress,
            updateBillingAddress,
            navigateNextStep,
            onUnhandledError,
            shippingAddress,
            billingAddress,
            methodId,
            isB3User,
            cart,
        } = this.props

        const updatedShippingAddress = addressValues && mapAddressFromFormValues(addressValues)
        const promises: Array<Promise<CheckoutSelectors>> = []
        const hasRemoteBilling = this.hasRemoteBilling(methodId)

        if (isB3User && B3Checkout.config.permissions['address.save']) {
            await B3Checkout.addresses.addNewB3Address({
                newAddress: updatedShippingAddress,
                billingSameAsShipping,
            })
        }

        if (!isEqualAddress(updatedShippingAddress, shippingAddress)) {
            promises.push(updateShippingAddress(updatedShippingAddress || {}))
        }

        if (billingSameAsShipping &&
            updatedShippingAddress &&
            !isEqualAddress(updatedShippingAddress, billingAddress) &&
            !hasRemoteBilling
        ) {
            if (
                !isB3User || 
                (
                    isB3User 
                    && B3Checkout.config.permissions['shipping.singleAddress.allowAsBilling']
                )
            ) promises.push(updateBillingAddress(updatedShippingAddress))
        }


console.log(cart);
console.log("testing shipping2");
const _global = (window /* browser */ || global /* node */) as any
_global.testRemoveProduct = function(cartId:string,itemId:string){
    var data = null;

    var xhr = new XMLHttpRequest();
    xhr.withCredentials = true;

    xhr.addEventListener("readystatechange", function () {
      if (this.readyState === this.DONE) {
        console.log(this.responseText);
      }
    });

    //var cartId = "ed5f4a03-8f88-4f26-802d-ced586aecbc9";
    //var itemId = "fddc71ae-4220-47f9-8f68-680daebdc557";
    var domain = window.location.hostname;

    xhr.open("DELETE", `https://${domain}/api/storefront/carts/${cartId}/items/${itemId}?include=lineItems.digitalItems.options%2ClineItems.physicalItems.options`);

    xhr.send(data);
}


        if (customerMessage !== orderComment) {
            promises.push(updateCheckout({ customerMessage: orderComment }))
        }

        try {
            await Promise.all(promises)

            navigateNextStep(billingSameAsShipping)
        } catch (error) {
            onUnhandledError(error)
        }
    };

    private hasRemoteBilling: (methodId?: string) => boolean = methodId => {
        const PAYMENT_METHOD_VALID = ['amazonpay']

        return PAYMENT_METHOD_VALID.some(method => method === methodId)
    };

    private handleUseNewAddress: (address: Address, itemId: string) => void = async (address, itemId) => {
        const { unassignItem, onUnhandledError } = this.props

        try {
            await unassignItem({
                shippingAddress: address,
                lineItems: [{
                    quantity: 1,
                    itemId,
                }],
            })

            location.href = '/account.php?action=add_shipping_address&from=checkout'
        } catch (e) {
            onUnhandledError(new UnassignItemError(e))
        }
    };

    private handleMultiShippingSubmit: (values: MultiShippingFormValues) => void = async ({ orderComment }) => {
        const {
            customerMessage,
            updateCheckout,
            navigateNextStep,
            onUnhandledError,
        } = this.props

        try {
            if (customerMessage !== orderComment) {
                await updateCheckout({ customerMessage: orderComment })
            }

            navigateNextStep(false)
        } catch (error) {
            onUnhandledError(error)
        }
    };
}

const deleteConsignmentsSelector = createSelector(
    ({ checkoutService: { deleteConsignment } }: CheckoutContextProps) => deleteConsignment,
    ({ checkoutState: { data } }: CheckoutContextProps) => data.getConsignments(),
    (deleteConsignment, consignments) => async () => {
        if (!consignments || !consignments.length) {
            return
        }

        const [{ data }] = await Promise.all(consignments.map(({ id }) =>
            deleteConsignment(id)
        ))

        return data.getShippingAddress()
    }
)

export function mapToShippingProps({
    checkoutService,
    checkoutState,
}: CheckoutContextProps): WithCheckoutShippingProps | null {
    const {
        data: {
            getCart,
            getCheckout,
            getConfig,
            getCustomer,
            getConsignments,
            getShippingAddress,
            getBillingAddress,
            getShippingAddressFields,
            getShippingCountries,
        },
        statuses: {
            isShippingStepPending,
            isSelectingShippingOption,
            isLoadingShippingOptions,
            isUpdatingConsignment,
            isCreatingConsignments,
            isLoadingShippingCountries,
            isUpdatingBillingAddress,
            isUpdatingCheckout,
        },
    } = checkoutState

    const checkout = getCheckout()
    const config = getConfig()
    const consignments = getConsignments() || []
    const customer = getCustomer()
    const cart = getCart()

    if (!checkout || !config || !customer || !cart) {
        return null
    }

    const {
        checkoutSettings: {
            enableOrderComments,
            features,
            hasMultiShippingEnabled,
            googleMapsApiKey,
        },
    } = config

    const methodId = getShippingMethodId(checkout)
    const shippableItemsCount = getShippableItemsCount(cart)
    const isLoading = (
        isLoadingShippingOptions() ||
        isSelectingShippingOption() ||
        isUpdatingConsignment() ||
        isCreatingConsignments() ||
        isUpdatingBillingAddress() ||
        isUpdatingCheckout()
    )
    const shouldShowMultiShipping = (
        hasMultiShippingEnabled &&
        !methodId &&
        shippableItemsCount > 1 &&
        shippableItemsCount < 50
    )
    const countriesWithAutocomplete = ['US', 'CA', 'AU', 'NZ']

    if (features['CHECKOUT-4183.checkout_google_address_autocomplete_uk']) {
        countriesWithAutocomplete.push('GB')
    }

    const shippingAddress = !shouldShowMultiShipping && consignments.length > 1 ? undefined : getShippingAddress()

    return {
        assignItem: checkoutService.assignItemsToAddress,
        billingAddress: getBillingAddress(),
        cart,
        consignments,
        countries: getShippingCountries() || EMPTY_ARRAY,
        countriesWithAutocomplete,
        customer,
        customerMessage: checkout.customerMessage,
        deinitializeShippingMethod: checkoutService.deinitializeShipping,
        deleteConsignments: deleteConsignmentsSelector({ checkoutService, checkoutState }),
        getFields: getShippingAddressFields,
        googleMapsApiKey,
        initializeShippingMethod: checkoutService.initializeShipping,
        isGuest: customer.isGuest,
        isInitializing: isLoadingShippingCountries() || isLoadingShippingOptions(),
        isLoading,
        isShippingStepPending: isShippingStepPending(),
        loadShippingAddressFields: checkoutService.loadShippingAddressFields,
        loadShippingOptions: checkoutService.loadShippingOptions,
        methodId,
        shippingAddress,
        shouldShowMultiShipping,
        shouldShowOrderComments: enableOrderComments,
        shouldValidateSafeInput: features['CHECKOUT-5327.validate_checkout_values'],
        signOut: checkoutService.signOutCustomer,
        unassignItem: checkoutService.unassignItemsToAddress,
        updateBillingAddress: checkoutService.updateBillingAddress,
        updateCheckout: checkoutService.updateCheckout,
        updateShippingAddress: checkoutService.updateShippingAddress,
    }
}

const mapStateToProps = (state: { global: { isB3User: boolean }, company: any }) => {
    const {
        global: {
            isB3User
        },
        company: b3Company
    } = state

    return {
        b3Company,
        isB3User,
    }
}

export default connect(mapStateToProps, {})(withCheckout(mapToShippingProps)(Shipping))
